@@ -170,10 +170,16 @@ impl<S: Stream<Item = Result<Message, ConnectionError>>> Future for Receiver<S>
                                 .get_mut(&consumer_id)
                                 .map(move |consumer| consumer.unbounded_send(msg));
                         }
-                        Some(RequestKey::CloseConsumer { consumer_id, request_id }) => {
+                        Some(RequestKey::CloseConsumer {
+                            consumer_id,
+                            request_id,
+                        }) => {
                             // FIXME: could the registration still be in queue while we get the
                             // CloseConsumer message?
-                            if let Some(resolver) = self.pending_requests.remove(&RequestKey::RequestId(request_id)) {
+                            if let Some(resolver) = self
+                                .pending_requests
+                                .remove(&RequestKey::RequestId(request_id))
+                            {
                                 // We don't care if the receiver has dropped their future
                                 let _ = resolver.send(msg);
                             } else {
@@ -186,7 +192,7 @@ impl<S: Stream<Item = Result<Message, ConnectionError>>> Future for Receiver<S>
                                     error!("ConnectionReceiver: error transmitting message to consumer: {:?}", res);
                                 }
                             }
-                        },
+                        }
                         None => {
                             warn!(
                                 "Received unexpected message; dropping. Message {:?}",
@@ -285,10 +291,27 @@ impl<Exe: Executor> ConnectionSender<Exe> {
                 .unbounded_send(Register::Ping { resolver }),
             self.tx.unbounded_send(messages::ping()),
         ) {
-            (Ok(_), Ok(_)) => response
-                .await
-                .map_err(|oneshot::Canceled| ConnectionError::Disconnected)
-                .map(move |_| trace!("received pong")),
+            (Ok(_), Ok(_)) => {
+                let delay_f = self.executor.delay(self.operation_timeout);
+                pin_mut!(response);
+                pin_mut!(delay_f);
+
+                match select(response, delay_f).await {
+                    Either::Left((res, _)) => res
+                        .map_err(|oneshot::Canceled| ConnectionError::Disconnected)
+                        .map(move |_| trace!("received pong")),
+                    Either::Right(_) => {
+                        self.error.set(ConnectionError::Io(std::io::Error::new(
+                            std::io::ErrorKind::TimedOut,
+                            "timeout when sending ping to the Pulsar server",
+                        )));
+                        Err(ConnectionError::Io(std::io::Error::new(
+                            std::io::ErrorKind::TimedOut,
+                            "timeout when sending ping to the Pulsar server",
+                        )))
+                    }
+                }
+            }
             _ => Err(ConnectionError::Disconnected),
         }
     }
@@ -809,6 +832,7 @@ impl<Exe: Executor> Connection<Exe> {
 
 impl<Exe: Executor> Drop for Connection<Exe> {
     fn drop(&mut self) {
+        trace!("dropping connection {} for {}", self.id, self.url);
         if let Some(shutdown) = self.sender.receiver_shutdown.take() {
             let _ = shutdown.send(());
         }
@@ -1066,7 +1090,7 @@ pub(crate) mod messages {
                         })
                         .collect(),
                     read_compacted: options.read_compacted,
-                    initial_position: options.initial_position,
+                    initial_position: Some(options.initial_position.into()),
                     schema: options.schema,
                     start_message_id: options.start_message_id,
                     ..Default::default()
